<?php

require_once dirname(__FILE__) . '/settings.php';

class GDPRNoticeAdmin {

	/**
	 * @var GDPRNoticeAdmin Class instance
	 */
	private static $instance;

	private static $tabs;

	public static function instance() {
		if (null == self::$instance) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	private function __construct() {
		add_action('admin_menu', array($this, 'addMenu'));
		add_action('admin_init', array($this, 'adminInit'));
		add_action('add_meta_boxes', array($this, 'addMetaBoxes'));
		add_filter('manage_gdpr-notice-page_posts_columns', array($this, 'addColumns'), 10, 1);
		add_action('manage_posts_custom_column', array($this, 'renderColumns'), 10, 2);
		add_action('load-edit.php', array($this, 'maybePrintColumnStyles'));

		add_filter('admin_title', array($this, 'adminTitle'), 10, 2);

		add_action('admin_post_gdpr_edit_style', array($this, 'saveStyle'));
		add_action('admin_enqueue_scripts', array($this, 'adminScripts'));
		add_action('admin_notices', array($this, 'adminNotices'));
	}

	public function init() {

	}

	public function adminInit() {
		add_action('pre_get_posts', array($this, 'sortByOrder'));
		add_action('save_post', array($this, 'save'), 10, 3);
		add_action('post_submitbox_start', array($this, 'renderNoUpdateCheckbox'));

		add_filter('parent_file', array($this, 'parentFile'));
		add_filter('submenu_file', array($this, 'submenuFile'));

		add_filter('display_post_states', array($this, 'postStates'), 10, 2);
		add_filter('post_row_actions', array($this, 'actionLinks'), 10, 2);
		add_action('post_action_toggle-disabled', array($this, 'toggleDisabled'));

		add_action('current_screen', array($this, 'queueTabs'));

		GDPRNoticeSettings::instance()->register();

		\de\orcas\extension\Nagger::instance()->registerRatingNag('gdpr-notice-original', 14);
		\de\orcas\extension\Nagger::instance()->registerProNag('gdpr-notice-original', 14);

		if (!session_id()) {
			session_start();
			if (!isset($_SESSION['gdpr-notice-original'])) {
				$_SESSION['gdpr-notice-original'] = array();
			}
		}
	}

	public function adminScripts() {
		if (function_exists('wp_enqueue_code_editor')) {
			if (isset($_GET['page']) && 'gdpr-style' == $_GET['page']) {
				$codeEditor = wp_enqueue_code_editor(array(
					'type' => 'text/css',
				));
				if (false !== $codeEditor) {
					wp_add_inline_script(
						'code-editor',
						sprintf(
							'jQuery( function() { wp.codeEditor.initialize( "gdpr-edit-style", %s ); } );',
							wp_json_encode($codeEditor)
						)
					);
					wp_add_inline_style('code-editor', '#gdpr-style-edit-form .CodeMirror { min-height: 60vh; height: calc(100vh - 360px); }');
				}
			}
		}
	}

	public function adminNotices() {
		if (isset($_SESSION['gdpr-notice-original']['message'])) {
			printf(
				'<div class="notice is-dismissible notice-%s"><p>%s</p></div>',
				$_SESSION['gdpr-notice-original']['type'],
				$_SESSION['gdpr-notice-original']['message']
			);
			unset($_SESSION['gdpr-notice-original']['message']);
			unset($_SESSION['gdpr-notice-original']['type']);
		}
	}

	public function queueTabs() {
		$currentScreen = get_current_screen();
		if (false !== ($pos = strpos($currentScreen->id, 'edit'))) {
			add_filter("views_{$currentScreen->id}", array($this, 'maybePrintTabs'));
		} else {
			add_action('all_admin_notices', array($this, 'maybePrintTabs'), 10000);
		}
	}

	/**
	 * @param WP_Query $query
	 */
	public function sortByOrder(&$query) {
		if ("gdpr-notice-page" == $query->query['post_type']) {
			if ('' == $query->get('orderby')) {
				$query->set('orderby', 'menu_order');
				$query->set('order', 'asc');
			}
		}
	}

	public function maybePrintColumnStyles() {
		if ("edit-gdpr-notice-page" == get_current_screen()->id) {
			add_action('admin_head', array($this, 'printColumnStyles'));
		}
	}

	public function printColumnStyles() { ?>
        <style type="text/css">
            .widefat .column-position {
                width: 60px;
                text-align: center;
            }
        </style>
	<?php }

	public function addColumns($columns) {
		$tmp = array();
		foreach ($columns as $name => $label) {
			$tmp[ $name ] = $label;
			if ("cb" == $name) {
				$tmp['position'] = __('Position', 'gdpr-notice-original');
			}
		}
		$tmp['page_type'] = __('Page type', 'gdpr-notice-original');

		return $tmp;
	}

	public function renderColumns($column, $postID) {
		switch ($column) {
			case 'position':
				$post = get_post($postID);
				echo $post->menu_order;
				break;
			case 'page_type':
				$pageTypes = apply_filters('gdpr_page_types', GDPRNotice::$pageTypes);
				$pageType  = get_post_meta($postID, 'page_type', true);
				if (isset($pageTypes[ $pageType ])) {
					echo $pageTypes[ $pageType ];
				} else {
					echo $pageType;
				}
				break;
		}
	}

	public function postStates($states, $post) {
		if ((!isset($_GET['post_status']) || "disabled" != $_GET['post_status']) && "disabled" == $post->post_status) {
			$states[] = _x('Disabled', 'post status', 'gdpr-notice-original');
		}

		return $states;
	}

	public function actionLinks($links, $post) {
		if ('gdpr-notice-page' == $post->post_type && (!isset($_GET['post_status']) || (isset($_GET['post_status']) && "trash" != $_GET['post_status']))) {
			$links = array_merge(
				array(
					'toggle-disabled' => sprintf(
						'<a href="%s">%s</a>',
						add_query_arg(
							array(
								'post'     => $post->ID,
								'action'   => 'toggle-disabled',
								'_wpnonce' => wp_create_nonce("toggle_disable_{$post->ID}")
							),
							admin_url('post.php')
						),
						"disabled" == $post->post_status ? __('Enable', 'gdpr-notice-original') : __('Disable', 'gdpr-notice-original')
					),
				),
				$links
			);
		}

		return $links;
	}

	public function toggleDisabled($postID) {
		if (!wp_verify_nonce($_REQUEST['_wpnonce'], "toggle_disable_{$postID}")) {
			wp_die(__('Oops, seems like something is wrong', 'gdpr-notice-original'));
		}
		$post              = get_post($postID);
		$post->post_status = 'publish' == $post->post_status ? 'disabled' : 'publish';
		wp_update_post($post);

		$back = wp_get_referer();
		wp_redirect(add_query_arg(array('post_type' => 'gdpr-notice-page'), $back));
		exit();
	}

	public function addMenu() {
		// orcas menu is added by /include/Upgrade/Upgrade.php:140
		add_submenu_page(
			'orcas',
			__('GDPR', 'gdpr-notice-original'),
			__('GDPR', 'gdpr-notice-original'),
			'manage_options',
			'edit.php?post_type=gdpr-notice-page'
		);

		// Dummy page for settings tab
		add_submenu_page(
			null,
			__('GDPR settings', 'gdpr-notice-original'),
			__('GDPR settings', 'gdpr-notice-original'),
			'manage_options',
			'gdpr-settings',
			array($this, 'renderSettingsPage')
		);

		// Dummy page for style tab
		add_submenu_page(
			null,
			__('GDPR style', 'gdpr-notice-original'),
			__('GDPR style', 'gdpr-notice-original'),
			'manage_options',
			'gdpr-style',
			array($this, 'renderStylePage')
		);
	}

	public function parentFile($file) {
		global $plugin_page, $current_screen;
		$tabs = $this->getTabs();
		foreach ($tabs as $tab) {
			if (in_array($current_screen->id, $tab['screens'])) {
				$plugin_page = 'orcas';
				break;
			}
		}

		return $file;
	}

	public function submenuFile($file) {
		global $plugin_page, $current_screen;
		if ('orcas' == $plugin_page) {
			$tabs = $this->getTabs();
			foreach ($tabs as $tab) {
				if (in_array($current_screen->id, $tab['screens'])) {
					$file = "edit.php?post_type=gdpr-notice-page";
					break;
				}
			}
		}

		return $file;
	}

	public function adminTitle($adminTitle, $title) {
		$title = false;
		switch (get_current_screen()->id) {
			case 'edit-gdpr-notice-page':
				$title = __('GDPR notice pages', 'gdpr-notice-original');
				break;
			case 'gdpr-notice-page':
				$title = __('Edit GDPR notice page', 'gdpr-notice-original');
				break;
			case 'admin_page_gdpr-settings':
				$title = __('GDPR settings', 'gdpr-notice-original');
				break;
			case 'admin_page_gdpr-style':
				$title = __('GDPR style', 'gdpr-notice-original');
				break;
			default:
				break;
		}
		if ($title) {
			$adminTitle = sprintf(__('%1$s &lsaquo; %2$s &#8212; WordPress'), $title, get_bloginfo('name'));
		}

		return $adminTitle;
	}

	public function addMetaBoxes() {
		add_meta_box('page-type', __('Page type', 'gdpr-notice-original'), array($this, 'renderMetaBox'), 'gdpr-notice-page', 'normal');
	}

	public function renderNoUpdateCheckbox($post) {
		if ("gdpr-notice-page" !== $post->post_type) return;
		?>
        <div id="do-not-update-notice-timestamp">
            <input type="checkbox" name="do_not_update_notice" id="do_not_update_notice"
                   title="<?php _e('If checked users will not have to re-accept the terms', 'gdpr-notice-original'); ?>"/>
            <label for="do_not_update_notice"
                   title="<?php _e('If checked users will not have to re-accept the terms', 'gdpr-notice-original'); ?>"><?php _e('Minor update', 'gdpr-notice-original'); ?></label>
        </div>
	<?php }

	public function renderMetaBox($post) {
		wp_nonce_field('_gdpr_notice_settings', '_gdpr_notice_nonce');
		echo '<select name="orcas_privacy_page-type" id="orcas_privacy_page-type">';
		$pageTypes    = apply_filters('gdpr_page_types', GDPRNotice::$pageTypes);
		$selectedType = get_post_meta($post->ID, 'page_type', true);
		foreach ($pageTypes as $pageType => $display) {
			echo "<option value='{$pageType}' " . selected($pageType, $selectedType, false) . ">{$display}</option>";
		}
		echo '</select>';
		do_action('gdpr_render_metabox', $post);
	}

	public function renderSettingsPage() {
		echo '<form method="post" action="options.php">';
		do_settings_sections('gdpr-settings');
		settings_fields('gdpr-settings');
		submit_button();
		echo '</form>';
	}

	public function getStyle() {
		$uplodas = wp_get_upload_dir();
		if (file_exists($uplodas['basedir'] . "/gdpr-notice/style.css")) {
			return file_get_contents($uplodas['basedir'] . "/gdpr-notice/style.css");
		} elseif (file_exists(dirname(__FILE__) . "/../view/assets/css/style.css")) {
			return file_get_contents(dirname(__FILE__) . "/../view/assets/css/style.css");
		}

		return "/*********************************/\n" .
		       " *  Something is severely wrong  *\n" .
		       " *********************************/";
	}

	public function renderStylePage() {
		if (!current_user_can('manage_options')) {
			wp_die(__('You have no permission to do this.', 'gdpr-notice-original'));
		}
		echo '<div class="wrap"><form method="post" action="admin-post.php" id="gdpr-style-edit-form">';
		wp_nonce_field('gdpr_edit_style', '_gdpr_edit_nonce');
		echo '<input type="hidden" name="action" value="gdpr_edit_style" />';
		echo '<textarea name="gdpr-edit-style" id="gdpr-edit-style" style="width:100%; height: 170ch;">';
		if (isset($_SESSION['gdpr-notice-original']['edited_style'])) {
			echo $_SESSION['gdpr-notice-original']['edited_style'];
			unset($_SESSION['gdpr-notice-original']['edited_style']);
		} else {
			echo $this->getStyle();
		}
		echo '</textarea>';
		submit_button(__('Save', 'gdpr-notice-original'));
		echo '</form>';
	}

	public function saveStyle() {
		if (!isset($_POST['_gdpr_edit_nonce']) || !wp_verify_nonce($_POST['_gdpr_edit_nonce'], 'gdpr_edit_style')) {
			wp_die(__('Nonce validation failed!', 'gdpr-notice-original'), __('Error', 'gdr-notice-original'), array(
				'response'  => 403,
				'back_link' => 'admin.php?page=gdpr-style',
			));
		}
		if (empty($_POST['gdpr-edit-style'])) {
			wp_die(__('Empty content!', 'gdpr-notice-original'), __('Error', 'gdr-notice-original'), array(
				'response'  => 403,
				'back_link' => 'admin.php?page=gdpr-style',
			));
		}
		$content = wp_unslash($_POST['gdpr-edit-style']);
		$session = &$_SESSION['gdpr-notice-original'];
		$uploads = wp_get_upload_dir();
		if (!is_dir($uploads['basedir'] . "/gdpr-notice/")) {
			mkdir($uploads['basedir'] . "/gdpr-notice/");
		}
		$file   = $uploads['basedir'] . "/gdpr-notice/style.css";
		$handle = fopen($file, 'w+');
		if (false === $handle) {
			$session['edited_style'] = $content;
			$session['message']      = __('File not writable', 'gdpr-notice-original');
			$session['type']         = 'error';
		} else {
			if (false === fwrite($handle, $content)) {
				$session['message'] = __('Error writing file', 'gdpr-notice-original');
				$session['type']    = 'error';
			} else {
				$session['message'] = __('File updated', 'gdpr-notice-original');
				$session['type']    = 'success';
			}
			@fclose($handle);
		}
		wp_safe_redirect(admin_url('admin.php?page=gdpr-style'));
	}

	private function getTabs() {
		if (empty(static::$tabs)) {
			static::$tabs = apply_filters('gdpr_admin_tabs', array(
				array(
					'title'   => __('Notice pages', 'gdpr-notice-original'),
					'screens' => array(
						'edit-gdpr-notice-page',
						'gdpr-notice-page'
					),
					'slug'    => 'edit.php?post_type=gdpr-notice-page',
				),
				array(
					'title'   => __('Settings', 'gdpr-notice-original'),
					'screens' => array(
						'admin_page_gdpr-settings'
					),
					'slug'    => 'admin.php?page=gdpr-settings',
				),
				array(
					'title'   => __('Style Editor', 'gdpr-notice-original'),
					'screens' => array(
						'admin_page_gdpr-style'
					),
					'slug'    => 'admin.php?page=gdpr-style',
				),
			));
		}

		return static::$tabs;
	}

	public function maybePrintTabs($views = null) {
		$currentScreen = get_current_screen();
		$tabs          = $this->getTabs();
		$found         = false;
		foreach ($tabs as $tab) {
			if (in_array($currentScreen->id, $tab['screens'])) {
				$found = true;
				break;
			}
		}
		if (!$found) {
			return $views;
		}
		echo '<h2 class="nav-tab-wrapper">';
		foreach ($tabs as $tab) {
			$active = in_array($currentScreen->id, $tab['screens']);
			printf('<a class="nav-tab %s" href="%s">%s</a>', $active ? 'nav-tab-active' : '', esc_url(admin_url($tab['slug'])), $tab['title']);
		}
		echo '</h2>';

		return $views;
	}

	public function save($postID, $post, $updated) {
		if ("gdpr-notice-page" == $post->post_type) {
			if (isset($_REQUEST['_gdpr_notice_nonce']) && wp_verify_nonce($_REQUEST['_gdpr_notice_nonce'], '_gdpr_notice_settings')) {
				$pageType = sanitize_text_field($_REQUEST['orcas_privacy_page-type']);
				if (in_array($pageType, array_keys(apply_filters('gdpr_page_types', GDPRNotice::$pageTypes)))) {
					// Only update post meta if page type exists
					update_post_meta($postID, 'page_type', $pageType);
				}
				$locale = explode('_', get_locale());
				$locale = $locale[0];
				if (!class_exists('GDPRNoticePro')) {
					// Only update language here if pro is not installed
					// Free does not use languages, but keep track of the language for when pro is installed
					update_post_meta($postID, 'language', $locale);
				}
			}
			if (!isset($_REQUEST['do_not_update_notice'])) {
				update_post_meta($postID, 'version', time());
			}
		}
	}
}

GDPRNoticeAdmin::instance();