<?php

class GDPRNoticeSettings {

	/**
	 * @var GDPRNoticeSettings Class instance
	 */
	private static $instance;

	public static function instance() {
		if (null == self::$instance) {
			self::$instance = new self();
		}

		return self::$instance;
	}

	public function register() {
		add_settings_section(
			'gdpr-notice-original_general',
			__('General settings', 'gdpr-notice-original'),
			'__return_null',
			'gdpr-settings'
		);

		add_settings_field(
			'gdpr-custom-robots',
			__('Custom robot user agents', 'gdpr-notice-original'),
			array($this, 'renderSettingsField'),
			'gdpr-settings',
			'gdpr-notice-original_general',
			array('id' => 'user-agents')
		);
		register_setting('gdpr-settings', 'gdpr-custom-robots', array(
			'sanitize_callback' => array($this, 'sanitizeRobots'),
		));

		add_settings_field(
			'gdpr-use-custom-url',
			__('Use custom URL', 'gdpr-notice-original'),
			array($this, 'renderSettingsField'),
			'gdpr-settings',
			'gdpr-notice-original_general',
			array('id' => 'use-custom-url')
		);
		register_setting('gdpr-settings', 'gdpr-use-custom-url', array(
			'sanitize_callback' => array($this, 'booleanize'),
		));

		add_settings_field(
			'gdpr-custom-url',
			__('Custom URL', 'gdpr-notice-original'),
			array($this, 'renderSettingsField'),
			'gdpr-settings',
			'gdpr-notice-original_general',
			array('id' => 'custom-url')
		);
		register_setting('gdpr-settings', 'gdpr-custom-url', array(
			'sanitize_callback' => 'wp_validate_redirect',
		));

		add_settings_field(
			'gdpr-background-image',
			__('Background image', 'gdpr-notice-original'),
			array($this, 'renderSettingsField'),
			'gdpr-settings',
			'gdpr-notice-original_general',
			array('id' => 'background-image')
		);
		register_setting('gdpr-settings', 'gdpr-background-image', array(
			'sanitize_callback' => 'absint',
		));
	}

	public function renderLicenseTitle() {
		_e('Manage your GDPR notice license here', 'gdpr-notice-original');
	}

	public function renderSettingsField($args) {
		switch ($args['id']) {
			case 'user-agents':
				echo '<textarea rows=6 name="gdpr-custom-robots" id="gdpr-custom-robots">';
				echo esc_html(join("\n", get_option('gdpr-custom-robots', array())));
				echo '</textarea><p>';
				_e('Enter custom regular expression to match the user agent against. The user is considered a bot if their user agent matches one (or more) of the default or your custom regular expressions and will not be asked to give their consent. One regular expression per line.', 'gdpr-notice-original');
				echo '</p>';
				break;
			case 'use-custom-url':
				echo '<input type="checkbox" name="gdpr-use-custom-url" id="gdpr-use-custom-url" value="on" ' . checked('yes', get_option('gdpr-use-custom-url', 'no'), false) . '/><p>';
				echo nl2br(__("By default, GDPR will output it's own content instead of the content of the requested page. This may cause issues with caching solutions.\nIf you check this box, GDPR will redirect the user to a custom URL and upon them confirming their consent redirect them back to the originally requested page.", 'gdpr-notice-original'));
				echo '</p>';
				break;
			case 'custom-url':
				echo trailingslashit(home_url());
				echo '<input type="hidden" name="gdpr-custom-url" value="' . esc_attr(get_option('gdpr-custom-url')) . '" />';
				echo '<input type="text" name="gdpr-custom-url" id="gdpr-custom-url" value="' . esc_attr(get_option('gdpr-custom-url')) . '" />';
				echo '<script>(function($){var c=$("#gdpr-use-custom-url");var s=$("#gdpr-custom-url");c.on("change",function(){s.prop("disabled",!$(this).prop("checked"));}).trigger("change");$("#gdpr_display_mode").on("change", function(){var d="normal"!=$(this).val();c.prop("disabled",d);s.prop("disabled",d||!c.is(":checked"));}).trigger("change");})(jQuery);</script>';
				break;
			case 'background-image':
				wp_enqueue_media();
				$image    = get_option('gdpr-background-image', "0");
				$imageSrc = array("");
				if ("0" != $image) {
					$imageSrc = wp_get_attachment_image_src($image, 'medium');

				}
				printf('<img id="gdpr-background-image-preview" src="%s" /><br/>', esc_url($imageSrc[0]));
				printf('<input type="hidden" name="gdpr-background-image" id="gdpr-background-image" value="%d">', esc_attr($image));
				printf('<input type="button" name="upload-btn" id="upload-btn" class="button-primary" value="%s">', esc_attr__('Select Image', 'gdpr-notice-original'));
				printf('<input type="button" name="delete-btn" id="delete-btn" class="delete button-secondary" value="%s" %s>', esc_attr__('Remove Image', 'gdpr-notice-original'), disabled("0", $image, false));
				?>
                <script>
                    jQuery(document).ready(function ($) {
                        var image;
                        var $delete = $("#delete-btn").on("click", function () {
                            $("#gdpr-background-image").val("");
                            $("#gdpr-background-image-preview").attr("src", "");
                            $(this).prop("disabled", true);
                        });
                        $("#upload-btn").click(function (e) {
                            e.preventDefault();
                            if (image) {
                                image.open();
                                return;
                            }
                            image = wp.media({
                                title: "<?php _e('Upload', 'gdpr-notice-original'); ?>",
                                multiple: false,
                                library: {type: "image"}
                            }).on("select", function (e) {
                                var uploaded_image = image.state().get("selection").first().toJSON();
                                var image_url = uploaded_image.sizes.medium.url;
                                var image_id = uploaded_image.id;
                                $("#gdpr-background-image").val(image_id);
                                $("#gdpr-background-image-preview").attr("src", image_url);
                                $delete.prop("disabled", false);
                            }).on("open", function () {
                                var selection = image.state().get("selection");
                                var attachment = wp.media.attachment($("#gdpr-background-image").val());
                                attachment.fetch();
                                selection.add(attachment ? [attachment] : []);
                            });
                            image.open();
                        });
                    });
                </script>
				<?php
				break;
		}
	}

	public function booleanize($val) {
		if ("on" === $val || "yes" === $val) {
			return 'yes';
		}

		return 'no';
	}

	public function sanitizeRobots($in) {
		$out = array();
		if (!is_array($in)) {
			if (is_string($in)) {
				$in = explode("\n", esc_textarea($in));
			} else {
				// Bail if we did not get an array or string.
				return $out;
			}
		}
		foreach ($in as $row) {
			if (!empty($row = trim($row))) {
				$out[] = str_replace('/', '\\/', $row);
			}
		}

		return $out;
	}
}

GDPRNoticeSettings::instance();